/************************************************************************
	main.c

	WFF USB Generic HID Demonstration 3
    usbGenericHidCommunication reference firmware 3_0_0_0
    Copyright (C) 2011 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#ifndef MAIN_C
#define MAIN_C

// Global includes
// Note: string.h is required for sprintf commands for debug
#include <string.h>

// Local includes
#include "HardwareProfile.h"
#include "debug.h"

// Microchip Application Library includes
// (expects V2.9a of the USB library from "Microchip Solutions v2011-07-14")
//
// The library location must be set in:
// Project -> Build Options Project -> Directories -> Include search path
// in order for the project to compile.
#include "./USB/usb.h"
#include "./USB/usb_function_hid.h"

// SPI libs
#include <spi.h>  //Include C18 SPI Library

#include <delays.h> //delay Library

#include <timers.h>

// Ensure we have the correct target PIC device family
#if !defined(__18F4550) && !defined(__18F2550) && !defined(__18F2450) 
	#error "This firmware only supports either the PIC18F4550 or PIC18F2550 microcontrollers."
#endif

// Define the globals for the USB data in the USB RAM of the PIC18F*550
#pragma udata
#pragma udata USB_VARIABLES=0x500 
unsigned char ReceivedDataBuffer[64];
unsigned char ToSendDataBuffer[64];
#pragma udata

USB_HANDLE USBOutHandle = 0;
USB_HANDLE USBInHandle = 0;
BOOL blinkStatusValid = FLAG_TRUE;

BOOL wait_timer = FLAG_TRUE;
unsigned int temp=0;

// PIC18F2450 configuration for the WFF Generic HID test device
#pragma config PLLDIV   = 5         // 20Mhz external oscillator
#pragma config CPUDIV   = OSC1_PLL2   
#pragma config USBDIV   = 2         // Clock source from 96MHz PLL/2
#pragma config FOSC     = HSPLL_HS
#pragma config FCMEN    = OFF
#pragma config IESO     = ON      // Just to verify LED
#pragma config PWRT     = ON
    //enable Power UP delay timer
#pragma config BOR      = ON
//#pragma config BORV     = 3
#pragma config VREGEN   = ON
#pragma config WDT      = OFF
#pragma config WDTPS    = 32768
#pragma config MCLRE    = OFF
#pragma config LPT1OSC  = OFF
#pragma config PBADEN   = OFF
// #pragma config CCP2MX   = ON
#pragma config STVREN   = ON
#pragma config LVP      = OFF
// #pragma config ICPRT    = OFF
#pragma config XINST    = OFF
#pragma config CP0      = OFF
#pragma config CP1      = OFF
// #pragma config CP2      = OFF
// #pragma config CP3      = OFF
#pragma config CPB      = OFF
// #pragma config CPD      = OFF
#pragma config WRT0     = OFF
#pragma config WRT1     = OFF
// #pragma config WRT2     = OFF
// #pragma config WRT3     = OFF
#pragma config WRTB     = OFF
#pragma config WRTC     = OFF
// #pragma config WRTD     = OFF
#pragma config EBTR0    = OFF
#pragma config EBTR1    = OFF
// #pragma config EBTR2    = OFF
// #pragma config EBTR3    = OFF
#pragma config EBTRB    = OFF

// Private function prototypes
static void initialisePic(void);
void processUsbCommands(void);
void applicationInit(void);
void USBCBSendResume(void);
void highPriorityISRCode();
void lowPriorityISRCode();
void strobe_LED(unsigned char, unsigned char, unsigned char, unsigned char);
void timer1_isr();
void update_pattern(void);

// Remap vectors for compatibilty with Microchip USB boot loaders
#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x1000
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x1008
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x1018
#elif defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)	
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x800
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x808
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x818
#else	
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x00
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x08
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x18
#endif

#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER) || defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
	extern void _startup (void);
	#pragma code REMAPPED_RESET_VECTOR = REMAPPED_RESET_VECTOR_ADDRESS
	void _reset (void)
	{
	    _asm goto _startup _endasm
	}
#endif

#pragma code REMAPPED_HIGH_INTERRUPT_VECTOR = REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS
void Remapped_High_ISR (void)
{
     _asm goto highPriorityISRCode _endasm
}

#pragma code REMAPPED_LOW_INTERRUPT_VECTOR = REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS
void Remapped_Low_ISR (void)
{
     _asm goto lowPriorityISRCode _endasm
}

#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER) || defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
#pragma code HIGH_INTERRUPT_VECTOR = 0x08
void High_ISR (void)
{
     _asm goto REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS _endasm
}

#pragma code LOW_INTERRUPT_VECTOR = 0x18
void Low_ISR (void)
{
     _asm goto REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS _endasm
}
#endif

#pragma code

// High-priority ISR handling function
#pragma interrupt highPriorityISRCode
void highPriorityISRCode()
{
   // Application specific high-priority ISR code goes here
   if (PIR2bits.TMR3IF){  // Interrupt Check 	
      wait_timer = FLAG_FALSE;
//      WriteTimer3(0x2B84);
      PIR2bits.TMR3IF = 0;
   }

   #if defined(USB_INTERRUPT)
		// Perform USB device tasks
		USBDeviceTasks();
   #endif

}

// Low-priority ISR handling function
#pragma interruptlow lowPriorityISRCode
void lowPriorityISRCode()
{
	// Application specific low-priority ISR code goes here
    wait_timer = FLAG_FALSE;
}

// String for creating debug messages
char debugString[80];

unsigned char strobe = 0;
unsigned char intensity = 0;
BOOL run = FLAG_TRUE;

typedef struct {
   unsigned char red;
   unsigned char green;
   unsigned char blue;
} strobes;

#pragma udata
#pragma udata LED_VARIABLES=0x200 
strobes led_data[64];
#pragma udata

unsigned char p_table=0, p_intensity=0,table_type=0,cycle_count =0,pattern_speed=0;
unsigned int p_count=0;
BOOL p_up_down=FLAG_FALSE;

// Main program entry point
void main(void)
{   
    unsigned char red_out, green_out, blue_out;
    unsigned char delay=0;

	// Initialise and configure the PIC ready to go
    initialisePic();

    // Show that we are up and running
//    mStatusLED0_on();

	// If we are running in interrupt mode attempt to attach the USB device
    #if defined(USB_INTERRUPT)
        USBDeviceAttach();
    #endif
	
	// Initialise the debug log functions
    debugInitialise();
	
	sprintf(debugString, "2DKits: 4x4x4 cube and 4x4 USB interface");
	debugOut(debugString);
	
	sprintf(debugString, "USB Device Initialised.");
	debugOut(debugString);
	
	// Main processing loop
    while(1)
    {
        #if defined(USB_POLLING)
			// If we are in polling mode the USB device tasks must be processed here
			// (otherwise the interrupt is performing this task)
	        USBDeviceTasks();
        #endif
    	
    	// Process USB Commands
        processUsbCommands();  

        // Note: Other application specific actions can be placed here
         if ((wait_timer == 0) && (run == TRUE)) { 
            wait_timer = 1;
            delay++;
            if (delay > pattern_speed) {
               delay = 0; 
               update_pattern();
            }
         }
         intensity = (intensity + 1) % 8;
         if (intensity == 0) {
            strobe = (strobe + 8) % 64;
         }
         red_out = 0x00;
         green_out = 0x00;
         blue_out = 0x00;
         if (led_data[strobe+0].red >intensity) { red_out += 0b00000001;}
         if (led_data[strobe+0].green >intensity) { green_out += 0b00000001;}
         if (led_data[strobe+0].blue >intensity) { blue_out += 0b00000001;}

         if (led_data[strobe+1].red >intensity) { red_out += 0b00000010;}
         if (led_data[strobe+1].green >intensity) { green_out += 0b00000010;}
         if (led_data[strobe+1].blue >intensity) { blue_out += 0b00000010;}

         if (led_data[strobe+2].red >intensity) { red_out += 0b00000100;}
         if (led_data[strobe+2].green >intensity) { green_out += 0b00000100;}
         if (led_data[strobe+2].blue >intensity) { blue_out += 0b00000100;}

         if (led_data[strobe+3].red >intensity) { red_out += 0b00001000;}
         if (led_data[strobe+3].green >intensity) { green_out += 0b00001000;}
         if (led_data[strobe+3].blue >intensity) { blue_out += 0b00001000;}

         if (led_data[strobe+4].red >intensity) { red_out += 0b00010000;}
         if (led_data[strobe+4].green >intensity) { green_out += 0b00010000;}
         if (led_data[strobe+4].blue >intensity) { blue_out += 0b00010000;}

         if (led_data[strobe+5].red >intensity) { red_out += 0b00100000;}
         if (led_data[strobe+5].green >intensity) { green_out += 0b00100000;}
         if (led_data[strobe+5].blue >intensity) { blue_out += 0b00100000;}

         if (led_data[strobe+6].red >intensity) { red_out += 0b01000000;}
         if (led_data[strobe+6].green >intensity) { green_out += 0b01000000;}
         if (led_data[strobe+6].blue >intensity) { blue_out += 0b01000000;}

         if (led_data[strobe+7].red >intensity) { red_out += 0b10000000;}
         if (led_data[strobe+7].green >intensity) { green_out += 0b10000000;}
         if (led_data[strobe+7].blue >intensity) { blue_out += 0b10000000;}

         strobe_LED(red_out, green_out, blue_out, (strobe>>3) );

    } // while   
}

#include "patterns_4x4x4.c"

unsigned char fader_cycle, fader_save ;

#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))

unsigned int up_down( unsigned char change, unsigned char data) {

  if (change) {
      return(MIN(8,data+1));
  } else {
      return(MAX(0,(signed char) data-1));
  }
}

void update_pattern() {

   unsigned char i,temp_in,temp_red,temp_green,temp_blue;
   unsigned int led_row;
   unsigned char *led_pointer;

// pull extra info bytes off the front of the pattern;

   if (p_count == 0) {
      table_type = patterns[p_table][p_count++];
      pattern_speed = patterns[p_table][p_count++];
   }

   switch (table_type) {
   case 1:
      for (i=0;i<8;i++) {
         temp_in = patterns[p_table][p_count];
         led_row = i*8;
         led_data[led_row+0].red = (temp_in & 0x80) ? (0x07) : (0x00);
         led_data[led_row+1].red = (temp_in & 0x40) ? (0x07) : (0x00);
         led_data[led_row+2].red = (temp_in & 0x20) ? (0x07) : (0x00);
         led_data[led_row+3].red = (temp_in & 0x10) ? (0x07) : (0x00);
         led_data[led_row+4].red = (temp_in & 0x08) ? (0x07) : (0x00);
         led_data[led_row+5].red = (temp_in & 0x04) ? (0x07) : (0x00);
         led_data[led_row+6].red = (temp_in & 0x02) ? (0x07) : (0x00);
         led_data[led_row+7].red = (temp_in & 0x01) ? (0x07) : (0x00);
         p_count++;
         temp_in = patterns[p_table][p_count];
         led_data[led_row+0].green = (temp_in & 0x80) ? (0x07) : (0x00);
         led_data[led_row+1].green = (temp_in & 0x40) ? (0x07) : (0x00);
         led_data[led_row+2].green = (temp_in & 0x20) ? (0x07) : (0x00);
         led_data[led_row+3].green = (temp_in & 0x10) ? (0x07) : (0x00);
         led_data[led_row+4].green = (temp_in & 0x08) ? (0x07) : (0x00);
         led_data[led_row+5].green = (temp_in & 0x04) ? (0x07) : (0x00);
         led_data[led_row+6].green = (temp_in & 0x02) ? (0x07) : (0x00);
         led_data[led_row+7].green = (temp_in & 0x01) ? (0x07) : (0x00);
         p_count++;
         temp_in = patterns[p_table][p_count];
         led_data[led_row+0].blue = (temp_in & 0x80) ? (0x07) : (0x00);
         led_data[led_row+1].blue = (temp_in & 0x40) ? (0x07) : (0x00);
         led_data[led_row+2].blue = (temp_in & 0x20) ? (0x07) : (0x00);
         led_data[led_row+3].blue = (temp_in & 0x10) ? (0x07) : (0x00);
         led_data[led_row+4].blue = (temp_in & 0x08) ? (0x07) : (0x00);
         led_data[led_row+5].blue = (temp_in & 0x04) ? (0x07) : (0x00);
         led_data[led_row+6].blue = (temp_in & 0x02) ? (0x07) : (0x00);
         led_data[led_row+7].blue = (temp_in & 0x01) ? (0x07) : (0x00);
         p_count++;
      }
      break;
   case 2:
      if (fader_cycle == 0) {
         fader_save = p_count;
      }
      for (i=0;i<8;i++) {
         temp_red = patterns[p_table][p_count++];
         temp_green = patterns[p_table][p_count++];
         temp_blue = patterns[p_table][p_count++];
         led_row = i*8;
//       breaking "good programing rules" for speed!
         led_pointer = &led_data[led_row].red;
// each lines now 30...
         *led_pointer++ = up_down((temp_red & 0x80),*led_pointer);
         *led_pointer++ = up_down((temp_green & 0x80),*led_pointer);
         *led_pointer++ = up_down((temp_blue & 0x80),*led_pointer);
         *led_pointer++ = up_down((temp_red & 0x40),*led_pointer);
         *led_pointer++ = up_down((temp_green & 0x40),*led_pointer);
         *led_pointer++ = up_down((temp_blue & 0x40),*led_pointer);
         *led_pointer++ = up_down((temp_red & 0x20),*led_pointer);
         *led_pointer++ = up_down((temp_green & 0x20),*led_pointer);
         *led_pointer++ = up_down((temp_blue & 0x20),*led_pointer);
         *led_pointer++ = up_down((temp_red & 0x10),*led_pointer);
         *led_pointer++ = up_down((temp_green & 0x10),*led_pointer);
         *led_pointer++ = up_down((temp_blue & 0x10),*led_pointer);
         *led_pointer++ = up_down((temp_red & 0x08),*led_pointer);
         *led_pointer++ = up_down((temp_green & 0x08),*led_pointer);
         *led_pointer++ = up_down((temp_blue & 0x08),*led_pointer);
         *led_pointer++ = up_down((temp_red & 0x04),*led_pointer);
         *led_pointer++ = up_down((temp_green & 0x04),*led_pointer);
         *led_pointer++ = up_down((temp_blue & 0x04),*led_pointer);
         *led_pointer++ = up_down((temp_red & 0x02),*led_pointer);
         *led_pointer++ = up_down((temp_green & 0x02),*led_pointer);
         *led_pointer++ = up_down((temp_blue & 0x02),*led_pointer);
         *led_pointer++ = up_down((temp_red & 0x01),*led_pointer);
         *led_pointer++ = up_down((temp_green & 0x01),*led_pointer);
         *led_pointer++ = up_down((temp_blue & 0x01),*led_pointer);

// each lines makes 36 instructions         
//         led_data[led_row+0].red = up_down((temp_red & 0x80),led_data[led_row+0].red);
//         led_data[led_row+1].red = up_down((temp_red & 0x40),led_data[led_row+1].red);
//         led_data[led_row+2].red = up_down((temp_red & 0x20),led_data[led_row+2].red);
//         led_data[led_row+3].red = up_down((temp_red & 0x10),led_data[led_row+3].red);
//         led_data[led_row+4].red = up_down((temp_red & 0x08),led_data[led_row+4].red);
//         led_data[led_row+5].red = up_down((temp_red & 0x04),led_data[led_row+5].red);
//         led_data[led_row+6].red = up_down((temp_red & 0x02),led_data[led_row+6].red);
//         led_data[led_row+7].red = up_down((temp_red & 0x01),led_data[led_row+7].red);
//         led_data[led_row+0].green = up_down((temp_green & 0x80),led_data[led_row+0].green);
//         led_data[led_row+1].green = up_down((temp_green & 0x40),led_data[led_row+1].green);
//         led_data[led_row+2].green = up_down((temp_green & 0x20),led_data[led_row+2].green);
//         led_data[led_row+3].green = up_down((temp_green & 0x10),led_data[led_row+3].green);
//         led_data[led_row+4].green = up_down((temp_green & 0x08),led_data[led_row+4].green);
//         led_data[led_row+5].green = up_down((temp_green & 0x04),led_data[led_row+5].green);
//         led_data[led_row+6].green = up_down((temp_green & 0x02),led_data[led_row+6].green);
//         led_data[led_row+7].green = up_down((temp_green & 0x01),led_data[led_row+7].green);
//         led_data[led_row+0].blue = up_down((temp_blue & 0x80),led_data[led_row+0].blue);
//         led_data[led_row+1].blue = up_down((temp_blue & 0x40),led_data[led_row+1].blue);
//         led_data[led_row+2].blue = up_down((temp_blue & 0x20),led_data[led_row+2].blue);
//         led_data[led_row+3].blue = up_down((temp_blue & 0x10),led_data[led_row+3].blue);
//         led_data[led_row+4].blue = up_down((temp_blue & 0x08),led_data[led_row+4].blue);
//         led_data[led_row+5].blue = up_down((temp_blue & 0x04),led_data[led_row+5].blue);
//         led_data[led_row+6].blue = up_down((temp_blue & 0x02),led_data[led_row+6].blue);
//         led_data[led_row+7].blue = up_down((temp_blue & 0x01),led_data[led_row+7].blue);
      }
      cycle_count = patterns[p_table][p_count];
      p_count++;
      if (fader_cycle < 8) {
          p_count = fader_save;
          fader_cycle++;
      } else {
          fader_cycle = 0;
      }
      break;
   }

   if (p_count >= pattern_size[p_table]) {
      p_count = 0;
   }

   if (!sw0) { 
      p_table = (p_table + 1) % table_count ;
      p_count = 0;
      sprintf(debugString, "Up to pattern %d", p_table);
	  debugOut(debugString);

   } else if (!sw1) {
      p_table = (p_table - 1) % table_count ;
      p_count = 0;
      sprintf(debugString, "Down to pattern %d", p_table);
	  debugOut(debugString);
   } else {
      // no button held
   }

}
  



// Initialise the PIC
static void initialisePic(void)
{
    unsigned char dummySPIdata;
   unsigned char i;

    // PIC port set up --------------------------------------------------------

	// Default all pins to digital
    ADCON1 = 0x0F;                                                                                                                                                                                                                                                                                             

	// Configure ports as inputs (1) or outputs(0)
	TRISA = 0b00000000;
	TRISB = 0b00000000;
	TRISC = 0b00000000;


	// Clear all ports
	PORTA = 0b00000000;
	PORTB = 0b00000000;
	PORTC = 0b00000000;


	// If you have a VBUS sense pin (for self-powered devices when you
	// want to detect if the USB host is connected) you have to specify
	// your input pin in HardwareProfile.h
    #if defined(USE_USB_BUS_SENSE_IO)
    	tris_usb_bus_sense = INPUT_PIN;
    #endif
    
    // In the case of a device which can be both self-powered and bus-powered
    // the device must respond correctly to a GetStatus (device) request and
    // tell the host how it is currently powered.
    //
    // To do this you must device a pin which is high when self powered and low
    // when bus powered and define this in HardwareProfile.h
    #if defined(USE_SELF_POWER_SENSE_IO)
    	tris_self_power = INPUT_PIN;
    #endif

    // Application specific initialisation
    applicationInit();
    
    // Initialise the USB device
    USBDeviceInit();

    // setup the LED drivers and SPI buss
 #define LED_enable 		PORTBbits.RB3
 #define LED_enable_dir		TRISBbits.TRISB3
 #define LED_latch   		PORTBbits.RB2
 #define LED_latch_dir		TRISBbits.TRISB2

 #define SPI_DataOut 		PORTCbits.RC7
 #define SPI_DataIn			PORTBbits.RB0
 #define SPI_Clk 			PORTBbits.RB1
 #define SPI_DataOut_Dir 	TRISCbits.TRISC7
 #define SPI_DataIn_Dir 	TRISBbits.TRISB0
 #define SPI_Clk_Dir 		TRISBbits.TRISB1


 #define LED_row0 		    PORTAbits.RA0
 #define LED_row0_dir		TRISAbits.TRISA0
 #define LED_row1 		    PORTAbits.RA1
 #define LED_row1_dir		TRISAbits.TRISA1
 #define LED_row2 		    PORTAbits.RA2
 #define LED_row2_dir		TRISAbits.TRISA2
 #define LED_row3 		    PORTAbits.RA3
 #define LED_row3_dir		TRISAbits.TRISA3
 #define LED_row4 		    PORTBbits.RB4
 #define LED_row4_dir		TRISBbits.TRISB4
 #define LED_row5 		    PORTBbits.RB5
 #define LED_row5_dir		TRISBbits.TRISB5
 #define LED_row6 		    PORTBbits.RB6
 #define LED_row6_dir		TRISBbits.TRISB6
 #define LED_row7 		    PORTBbits.RB7
 #define LED_row7_dir		TRISBbits.TRISB7

	//Configure I/O for LED and SPI

	SPI_DataOut_Dir 	=0;
	SPI_DataIn_Dir 		=1;
	SPI_Clk_Dir 		=0;

    LED_enable_dir    =0;  //output
    LED_latch_dir    =0;  //output

    LED_row0_dir     =0;   //output
    LED_row1_dir     =0;   //output
    LED_row2_dir     =0;   //output
    LED_row3_dir     =0;   //output
    LED_row4_dir     =0;   //output
    LED_row5_dir     =0;   //output
    LED_row6_dir     =0;   //output
    LED_row7_dir     =0;   //output

    LED_row0         =0;   //off
    LED_row1         =0;   //off
    LED_row2         =0;   //off
    LED_row3         =0;   //off
    LED_row4         =0;   //off
    LED_row5         =0;   //off
    LED_row6         =0;   //off
    LED_row7         =0;   //off

    LED_enable        =0;  // set low to turn on drivers
    LED_latch        =0;  // latch off

    OpenSPI(SPI_FOSC_4,MODE_00,SMPMID);

    OpenTimer3(  TIMER_INT_ON &                // Interrupt enabled 
                 T3_16BIT_RW &                  //set timer1 as 16 bit registers 
                 T3_SOURCE_INT &               //choose internal clock source (TOSC) 
                 T3_PS_1_8 &                   // Prescale Value: 1:8 
                 T3_SYNC_EXT_OFF               //Dont sync external clock input 
              ); 

	RCONbits.IPEN=1;     //Enable priority levels on interrupts
	RCONbits.SBOREN=0;   //Disable BOR

    IPR2bits.TMR3IP = 1; //Timer3 interrupt priority hi 
    T3CONbits.TMR3ON = 1;	/* Enable the timer3 */
    PIE2bits.TMR3IE = 1;	/* Enable timer3 interrupt */	
    INTCONbits.GIEL = 1; //enable low pri interrupts


   for (i=0;i<64;i++) {
      led_data[i].red = 0;
      led_data[i].green = 0;
      led_data[i].blue = 0;
   }
}

void strobe_LED(unsigned char red, unsigned char green, unsigned char blue, unsigned char row )
{
static unsigned char old_row = 0;
//  write 24 bits of column data via SPI buss
    WriteSPI(green); // all green on
    WriteSPI(red); // all red on

// I turn off the rows, then send the blue to stop a pre-ghost issue I saw.
// My guess is the Source Driver chip needs some time to turn off before I latch
// the next column.

//  all rows off 
if (old_row!=row) {
    LED_row0         =0;  // why is this command not working?
    LED_row1         =0;
    LED_row2         =0;
    LED_row3         =0;
    LED_row4         =0;
    LED_row5         =0;
    LED_row6         =0;
    LED_row7         =0;
    LED_row0         =0;  // I have no idea why I have to do it a 2nd time!!
}
    WriteSPI(blue); // all blue on

//  latch the 24 bits of column data
    LED_latch        =1;  // latch the LED data
    LED_latch        =0;

if (old_row!=row) {
switch (row) {
       case 0:
          LED_row0         =1;   
          break;
       case 1:
          LED_row1         =1; 
          break;
       case 2:
          LED_row2         =1;
          break;
       case 3:
          LED_row3         =1;
          break;
       case 4:
          LED_row4         =1;
          break;
       case 5:
          LED_row5         =1;
          break;
       case 6:
          LED_row6         =1;
          break;
       case 7:
          LED_row7         =1;
          break;
       default:
          break;
   	}   
   }
   old_row = row;
}

// Application specific device initialisation
void applicationInit(void)
{
	// Initialise the status LEDs
	mInitStatusLeds();
	
	// Initialise the switch
	mInitAllSwitches();
	
    // Initialize the variable holding the USB handle for the last transmission
    USBOutHandle = 0;
    USBInHandle = 0;
}

// Process USB commands
void processUsbCommands(void)
{   
    // Check if we are in the configured state; otherwise just return
    if((USBDeviceState < CONFIGURED_STATE) || (USBSuspendControl == 1))
    {
	    // We are not configured
	    return;
	}

	// Check if data was received from the host.
    if(!HIDRxHandleBusy(USBOutHandle))
    {   
		// Command mode    
        switch(ReceivedDataBuffer[0])
		{
			case 0x10:	// Debug information request from host
				// Copy any waiting debug text to the send data buffer
				copyDebugToSendBuffer((BYTE*)&ToSendDataBuffer[0]);
			        
		        // Transmit the response to the host
                if(!HIDTxHandleBusy(USBInHandle))
				{
					USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
				}
				break;
				
			// Place application specific commands here:
			
            case 0x80:  // Toggle the LED
            	sprintf(debugString, "Received command 0x80 from host - Toggle LED %d %#04X", wait_timer, temp);
				debugOut(debugString);

				// Toggle the LED0
				mStatusLED0_Toggle();
                run = !run;
            	break;
            	
            case 0x81:  // Read the push switch status
				ToSendDataBuffer[0] = sw0 + (sw1<<1);

				// Transmit the response to the host
                if(!HIDTxHandleBusy(USBInHandle))
				{
					USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
				}
            	break;
            	
            case 0x82:  // Read the LED status
				// Get the LED state and put it in the send buffer
				ToSendDataBuffer[0] = mStatusLED0_Get();
				
				// Transmit the response to the host
                if(!HIDTxHandleBusy(USBInHandle))
				{
					USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
				}
            	break;

            case 0x83:  // RGB LED set command
				// process an RBG matrix command
		        led_data[ReceivedDataBuffer[4]].red = ReceivedDataBuffer[1];
		        led_data[ReceivedDataBuffer[4]].green = ReceivedDataBuffer[2];
		        led_data[ReceivedDataBuffer[4]].blue = ReceivedDataBuffer[3];
            	break;

            default:	// Unknown command received
           		break;
		}
        // Re-arm the OUT endpoint for the next packet
        USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
  	}
}

// USB Callback handling routines -----------------------------------------------------------

// Call back that is invoked when a USB suspend is detected
void USBCBSuspend(void)
{
}

// This call back is invoked when a wakeup from USB suspend is detected.
void USBCBWakeFromSuspend(void)
{
}

// The USB host sends out a SOF packet to full-speed devices every 1 ms.
void USBCB_SOF_Handler(void)
{
    // No need to clear UIRbits.SOFIF to 0 here. Callback caller is already doing that.
}

// The purpose of this callback is mainly for debugging during development.
// Check UEIR to see which error causes the interrupt.
void USBCBErrorHandler(void)
{
    // No need to clear UEIR to 0 here.
    // Callback caller is already doing that.
}

// Check other requests callback
void USBCBCheckOtherReq(void)
{
    USBCheckHIDRequest();
}

// Callback function is called when a SETUP, bRequest: SET_DESCRIPTOR request arrives.
void USBCBStdSetDscHandler(void)
{
    // You must claim session ownership if supporting this request
}

//This function is called when the device becomes initialized
void USBCBInitEP(void)
{
    // Enable the HID endpoint
    USBEnableEndpoint(HID_EP,USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
    
    // Re-arm the OUT endpoint for the next packet
    USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
}

// Send resume call-back
void USBCBSendResume(void)
{
    static WORD delay_count;
    
    // Verify that the host has armed us to perform remote wakeup.
    if(USBGetRemoteWakeupStatus() == FLAG_TRUE) 
    {
        // Verify that the USB bus is suspended (before we send remote wakeup signalling).
        if(USBIsBusSuspended() == FLAG_TRUE)
        {
            USBMaskInterrupts();
            
            // Bring the clock speed up to normal running state
            USBCBWakeFromSuspend();
            USBSuspendControl = 0; 
            USBBusIsSuspended = FLAG_FALSE;

            // Section 7.1.7.7 of the USB 2.0 specifications indicates a USB
            // device must continuously see 5ms+ of idle on the bus, before it sends
            // remote wakeup signalling.  One way to be certain that this parameter
            // gets met, is to add a 2ms+ blocking delay here (2ms plus at 
            // least 3ms from bus idle to USBIsBusSuspended() == FLAG_TRUE, yeilds
            // 5ms+ total delay since start of idle).
            delay_count = 3600U;        
            do
            {
                delay_count--;
            } while(delay_count);
            
            // Start RESUME signaling for 1-13 ms
            USBResumeControl = 1;
            delay_count = 1800U;
            do
            {
                delay_count--;
            } while(delay_count);
            USBResumeControl = 0;

            USBUnmaskInterrupts();
        }
    }
}

// USB callback function handler
BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size)
{
    switch(event)
    {
        case EVENT_TRANSFER:
            // Application callback tasks and functions go here
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        default:
            break;
    }      
    return FLAG_TRUE; 
}

#endif
